@@ -1,8 +1,9 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024-2025 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,13 +16,15 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
 #ifndef DOWNLOADINFO_H
 #define DOWNLOADINFO_H
 
+#include <atomic>
+#include "NewsServer.h"
 #include "NString.h"
 #include "Container.h"
 #include "Observer.h"
@@ -103,15 +106,15 @@ public:
 	void SetCrc(uint32 crc) { m_crc = crc; }
 
 private:
-	int m_partNumber;
-	CString m_messageId;
-	int m_size = 0;
 	std::unique_ptr<SegmentData> m_segmentContent;
-	int64 m_segmentOffset = 0;
-	int m_segmentSize = 0;
-	EStatus m_status = aiUndefined;
+	std::atomic<int64> m_segmentOffset{0};
+	std::atomic<int> m_segmentSize{0};
+	std::atomic<uint32> m_crc{0};
+	CString m_messageId;
 	CString m_resultFilename;
-	uint32 m_crc = 0;
+	EStatus m_status = aiUndefined;
+	int m_partNumber;
+	int m_size = 0;
 };
 
 typedef std::vector<std::unique_ptr<ArticleInfo>> ArticleList;
@@ -138,8 +141,10 @@ public:
 	Groups* GetGroups() { return &m_groups; }
 	const char* GetSubject() { return m_subject; }
 	void SetSubject(const char* subject) { m_subject = subject; }
-	const char* GetFilename() { return m_filename; }
-	void SetFilename(const char* filename) { m_filename = filename; }
+	const char* GetFilename() { return m_filename.c_str(); }
+	void SetFilename(std::string filename) { m_filename = std::move(filename); }
+	void SetOrigname(const char* origname) { m_origname = origname; }
+	const char* GetOrigname() { return m_origname; }
 	void MakeValidFilename();
 	bool GetFilenameConfirmed() { return m_filenameConfirmed; }
 	void SetFilenameConfirmed(bool filenameConfirmed) { m_filenameConfirmed = filenameConfirmed; }
@@ -172,8 +177,9 @@ public:
 	bool GetParFile() { return m_parFile; }
 	void SetParFile(bool parFile) { m_parFile = parFile; }
 	Guard GuardOutputFile() { return Guard(m_outputFileMutex); }
-	const char* GetOutputFilename() { return m_outputFilename; }
-	void SetOutputFilename(const char* outputFilename) { m_outputFilename = outputFilename; }
+	const std::string& GetOutputFilename() const { return m_outputFilename; }
+	void SetOutputFilename(const char* outputFilename) { m_outputFilename = outputFilename ? outputFilename : ""; }
+	void SetOutputFilename(std::string outputFilename) { m_outputFilename = std::move(outputFilename); }
 	bool GetOutputInitialized() { return m_outputInitialized; }
 	void SetOutputInitialized(bool outputInitialized) { m_outputInitialized = outputInitialized; }
 	bool GetExtraPriority() { return m_extraPriority; }
@@ -196,6 +202,8 @@ public:
 	void SetHash16k(const char* hash16k) { m_hash16k = hash16k; }
 	const char* GetParSetId() { return m_parSetId; }
 	void SetParSetId(const char* parSetId) { m_parSetId = parSetId; }
+	bool GetFlushLocked() { return m_flushLocked; }
+	void SetFlushLocked(bool flushLocked) { m_flushLocked = flushLocked; }
 
 	ServerStatList* GetServerStats() { return &m_serverStats; }
 
@@ -206,7 +214,8 @@ private:
 	Groups m_groups;
 	ServerStatList m_serverStats;
 	CString m_subject;
-	CString m_filename;
+	std::string m_filename;
+	CString m_origname;
 	int64 m_size = 0;
 	int64 m_remainingSize = 0;
 	int64 m_successSize = 0;
@@ -223,18 +232,19 @@ private:
 	bool m_parFile = false;
 	int m_completedArticles = 0;
 	bool m_outputInitialized = false;
-	CString m_outputFilename;
+	std::string m_outputFilename;
 	std::unique_ptr<Mutex> m_outputFileMutex;
 	bool m_extraPriority = false;
 	int m_activeDownloads = 0;
 	bool m_dupeDeleted = false;
-	int m_cachedArticles = 0;
+	std::atomic<int> m_cachedArticles{0};
 	bool m_partialChanged = false;
 	bool m_forceDirectWrite = false;
 	EPartialState m_partialState = psNone;
 	uint32 m_crc = 0;
 	CString m_hash16k;
 	CString m_parSetId;
+	bool m_flushLocked = false;
 
 	static int m_idGen;
 	static int m_idMax;
@@ -256,27 +266,30 @@ public:
 		cfFailure
 	};
 
-	CompletedFile(int id, const char* filename, EStatus status, uint32 crc, 
-		bool parFile, const char* hash16k, const char* parSetId);
+	CompletedFile(int id, std::string filename, std::string oldname, EStatus status,
+		uint32 crc, bool parFile, std::string hash16k, std::string parSetId);
 	int GetId() { return m_id; }
-	void SetFilename(const char* filename) { m_filename = filename; }
-	const char* GetFilename() { return m_filename; }
+	void SetFilename(std::string filename) { m_filename = std::move(filename); }
+	const char* GetFilename() { return m_filename.c_str(); }
+	void SetOrigname(std::string origname) { m_origname = std::move(origname); }
+	const char* GetOrigname() { return m_origname.c_str(); }
 	bool GetParFile() { return m_parFile; }
 	EStatus GetStatus() { return m_status; }
 	uint32 GetCrc() { return m_crc; }
-	const char* GetHash16k() { return m_hash16k; }
-	void SetHash16k(const char* hash16k) { m_hash16k = hash16k; }
-	const char* GetParSetId() { return m_parSetId; }
-	void SetParSetId(const char* parSetId) { m_parSetId = parSetId; }
+	const char* GetHash16k() { return m_hash16k.c_str(); }
+	void SetHash16k(std::string hash16k) { m_hash16k = std::move(hash16k); }
+	const char* GetParSetId() { return m_parSetId.c_str(); }
+	void SetParSetId(std::string parSetId) { m_parSetId = std::move(parSetId); }
 
 private:
 	int m_id;
-	CString m_filename;
 	EStatus m_status;
 	uint32 m_crc;
 	bool m_parFile;
-	CString m_hash16k;
-	CString m_parSetId;
+	std::string m_filename;
+	std::string m_origname;
+	std::string m_hash16k;
+	std::string m_parSetId;
 };
 
 typedef std::deque<CompletedFile> CompletedFileList;
@@ -304,7 +317,7 @@ class NzbParameterList : public NzbParameterListBase
 {
 public:
 	void SetParameter(const char* name, const char* value);
-	NzbParameter* Find(const char* name, bool caseSensitive);
+	NzbParameter* Find(const char* name);
 	void CopyFrom(NzbParameterList* sourceParameters);
 };
 
@@ -406,6 +419,14 @@ public:
 		msSuccess
 	};
 
+	enum class PostUnpackRenamingStatus
+	{
+		None,
+		Failure,
+		Success,
+		Skipped
+	};
+
 	enum EDeleteStatus
 	{
 		dsNone,
@@ -443,6 +464,13 @@ public:
 		nkUrl
 	};
 
+	enum EDupeHint
+	{
+		dhNone,
+		dhRedownloadManual,
+		dhRedownloadAuto
+	};
+
 	int GetId() { return m_id; }
 	void SetId(int id);
 	static void ResetGenId(bool max);
@@ -535,6 +563,8 @@ public:
 	ECleanupStatus GetCleanupStatus() { return m_cleanupStatus; }
 	void SetCleanupStatus(ECleanupStatus cleanupStatus) { m_cleanupStatus = cleanupStatus; }
 	EMoveStatus GetMoveStatus() { return m_moveStatus; }
+	void SetPostUnpackRenamingStatus(PostUnpackRenamingStatus status) { m_postUnpackRenamingStatus = status; }
+	PostUnpackRenamingStatus GetPostUnpackRenamingStatus() { return m_postUnpackRenamingStatus; }
 	void SetMoveStatus(EMoveStatus moveStatus) { m_moveStatus = moveStatus; }
 	EDeleteStatus GetDeleteStatus() { return m_deleteStatus; }
 	void SetDeleteStatus(EDeleteStatus deleteStatus) { m_deleteStatus = deleteStatus; }
@@ -577,6 +607,8 @@ public:
 	void SetDupeScore(int dupeScore) { m_dupeScore = dupeScore; }
 	EDupeMode GetDupeMode() { return m_dupeMode; }
 	void SetDupeMode(EDupeMode dupeMode) { m_dupeMode = dupeMode; }
+	EDupeHint GetDupeHint() { return m_dupeHint; }
+	void SetDupeHint(EDupeHint dupeHint) { m_dupeHint = dupeHint; }
 	uint32 GetFullContentHash() { return m_fullContentHash; }
 	void SetFullContentHash(uint32 fullContentHash) { m_fullContentHash = fullContentHash; }
 	uint32 GetFilteredContentHash() { return m_filteredContentHash; }
@@ -612,7 +644,7 @@ public:
 	void LeavePostProcess();
 	bool IsDupeSuccess();
 	const char* MakeTextStatus(bool ignoreScriptStatus);
-	void AddMessage(Message::EKind kind, const char* text);
+	void AddMessage(Message::EKind kind, const char* text, bool print = true);
 	void PrintMessage(Message::EKind kind, const char* format, ...) PRINTF_SYNTAX(3);
 	int GetMessageCount() { return m_messageCount; }
 	void SetMessageCount(int messageCount) { m_messageCount = messageCount; }
@@ -630,6 +662,12 @@ public:
 	void UpdateCompletedStats(FileInfo* fileInfo);
 	void UpdateDeletedStats(FileInfo* fileInfo);
 	bool IsDownloadCompleted(bool ignorePausedPars);
+	void SetDesiredServerId(int id) { m_desiredServerId = id; }
+	int GetDesiredServerId() { return m_desiredServerId; }
+	bool GetSkipScriptProcessing() { return m_skipScriptProcessing; }
+	void SetSkipScriptProcessing(bool skip) { m_skipScriptProcessing = skip; }
+	void SetSkipDiskWrite(bool skipWrite) { m_scipDiskWrite = skipWrite; }
+	bool GetSkipDiskWrite() { return m_scipDiskWrite; }
 
 	static const int FORCE_PRIORITY = 900;
 
@@ -677,6 +715,7 @@ private:
 	EPostUnpackStatus m_unpackStatus = usNone;
 	ECleanupStatus m_cleanupStatus = csNone;
 	EMoveStatus m_moveStatus = msNone;
+	PostUnpackRenamingStatus m_postUnpackRenamingStatus = PostUnpackRenamingStatus::None;
 	EDeleteStatus m_deleteStatus = dsNone;
 	EMarkStatus m_markStatus = ksNone;
 	EUrlStatus m_urlStatus = lsNone;
@@ -695,6 +734,7 @@ private:
 	CString m_dupeKey = "";
 	int m_dupeScore = 0;
 	EDupeMode m_dupeMode = dmScore;
+	EDupeHint m_dupeHint = dhNone;
 	uint32 m_fullContentHash = 0;
 	uint32 m_filteredContentHash = 0;
 	FileList m_fileList;
@@ -725,6 +765,9 @@ private:
 	bool m_waitingPar = false;
 	bool m_loadingPar = false;
 	Thread* m_unpackThread = nullptr;
+	int m_desiredServerId = 0;
+	bool m_skipScriptProcessing = false;
+	bool m_scipDiskWrite = false;
 
 	static int m_idGen;
 	static int m_idMax;
@@ -752,6 +795,7 @@ public:
 		ptUnpacking,
 		ptCleaningUp,
 		ptMoving,
+		ptPostUnpackRenaming,
 		ptExecutingScript,
 		ptFinished
 	};
@@ -800,7 +844,7 @@ public:
 
 private:
 	NzbInfo* m_nzbInfo = nullptr;
-	bool m_working = false;
+	std::atomic<bool> m_working{false};
 	bool m_deleted = false;
 	bool m_requestParCheck = false;
 	bool m_forceParFull = false;
@@ -882,7 +926,7 @@ public:
 	};
 
 	HistoryInfo(std::unique_ptr<NzbInfo> nzbInfo) : m_info(nzbInfo.release()),
-		m_kind(nzbInfo->GetKind() == NzbInfo::nkNzb ? hkNzb : hkUrl) {}
+		m_kind(GetNzbInfo()->GetKind() == NzbInfo::nkNzb ? hkNzb : hkUrl) {}
 	HistoryInfo(std::unique_ptr<DupInfo> dupInfo) : m_info(dupInfo.release()), m_kind(hkDup) {}
 	~HistoryInfo();
 	EKind GetKind() { return m_kind; }
@@ -895,8 +939,8 @@ public:
 	const char* GetName();
 
 private:
-	EKind m_kind;
 	void* m_info;
+	EKind m_kind;
 	time_t m_time = 0;
 };
 
@@ -913,9 +957,15 @@ public:
 		eaNzbAdded,
 		eaNzbDeleted,
 		eaNzbNamed,
+		eaNzbReturned,
 		eaFileCompleted,
 		eaFileDeleted,
-		eaUrlCompleted
+		eaUrlFound,
+		eaUrlAdded,
+		eaUrlDeleted,
+		eaUrlCompleted,
+		eaUrlFailed,
+		eaUrlReturned
 	};
 
 	struct Aspect
@@ -996,6 +1046,7 @@ public:
 	virtual bool EditList(IdList* idList, NameList* nameList, EMatchMode matchMode, EEditAction action, const char* args) = 0;
 	virtual void HistoryChanged() = 0;
 	virtual void Save() = 0;
+	virtual void SaveChanged() = 0;
 	void CalcRemainingSize(int64* remaining, int64* remainingForced);
 
 protected:
@@ -1010,7 +1061,7 @@ private:
 	Mutex m_lockMutex;
 
 	static DownloadQueue* g_DownloadQueue;
-	static bool g_Loaded;
+	static std::atomic<bool> g_Loaded;
 };
 
 #endif
